package core

import (
	"git.akyoto.dev/cli/q/src/asm"
	"git.akyoto.dev/cli/q/src/expression"
)

// CompileNew compiles a `new` function call which allocates a struct.
func (f *Function) CompileNew(root *expression.Expression) error {
	parameters := root.Children[1:]
	structName := parameters[0].Token.Text(f.File.Bytes)
	typ := f.Types[structName]
	f.SaveRegister(f.CPU.Input[0])
	f.RegisterNumber(asm.MOVE, f.CPU.Input[0], typ.Size())

	for _, register := range f.CPU.General {
		if f.RegisterIsUsed(register) {
			f.Register(asm.PUSH, register)
		}
	}

	f.Call("mem.alloc")

	for i := len(f.CPU.General) - 1; i >= 0; i-- {
		register := f.CPU.General[i]

		if f.RegisterIsUsed(register) {
			f.Register(asm.POP, register)
		}
	}

	return nil
}
